/*
 * pcal2 - pcal3 Changes
 *    TITLE FONT, NUMBER FONT, KANJI FONT, and ASCII FONT are
 *  modifiable respectively.
 */

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include "kpcal_header.h"

#define PRT	(void)printf
#define FPR	(void)fprintf

char *words[100];	/* maximum number of words on a line */
char lbuf[512];		/* maximum line size */

char *months[] = {	/* used to match alpha months */
	"jan", "feb", "mar", "apr", "may", "jun",
	"jul", "aug", "sep", "oct", "nov", "dec",
	(char *)0,
};

static char ascii_font[64] = ASCII_DEFAULT_FONT;
static char kanji_font[64] = KANJI_DEFAULT_FONT;
static char title_font[64] = TITLE_DEFAULT_FONT;
static char daynum_font[64] = DAYNUM_DEFAULT_FONT;

extern void ConvertString();

FILE *cfp = NULL;
int year;
int cyear;

char *getenv();

main(argc, argv)
     int argc;
     char **argv;
{
  extern char *optarg;
  extern int optind;
  register struct tm *lt;
  register char **ap;
  register char *cp;
  char *cfile = NULL;
  char cbuf[80];
  long t, time();
  int errflg = 0;
  int nocal = 0;
  int m;

  while ((m = getopt(argc, argv, "a:ef:hk:n:t:?")) != EOF)
    switch (m) {
    case 'a': 
      strcpy(ascii_font,optarg);
      break;
    case 'e':
      nocal++;
      cfile = NULL;
      break;
    case 'f':
      cfile = optarg;
      nocal = 0;
      break;
    case 'h':
      errflg = 1;
      break;
    case 'k':
      strcpy(kanji_font,optarg);
      break;
    case 'n':
      strcpy(daynum_font,optarg);
      break;
    case 't':
      strcpy(title_font,optarg);
      break;
    case '?':
      errflg = 1;
      break;
    }
  if (errflg) {
    FPR(stderr, "Usage: kpcal [-a <ASCII Font>][-k <KANJI Font>]\
[-n <NUMBER Font>][-t <TITLE Font>][ -e | -f <cal>] [ month [year]]\n");
    exit(1);
  }

  t = time((long *)0);
  lt = localtime(&t);

  /*
   * First argument should be a month
   */
  if ((argc - optind) > 0) {
    if ((m = atoi(argv[optind])) < 1 || m > 12) {
      FPR(stderr, "Month must be in range 1 - 12\n");
      exit(1);
    }
  } else	/* otherwise, use current month */
    m = lt->tm_mon + 1;
  
  /*
   * More than one arg, second must be a year
   */
  if ((argc - optind) > 1) {
    if ((year = atoi(argv[optind+1])) < 100)
      year += 1900;
  } else
    year = lt->tm_year + 1900;
  
  /*
   * In case we don't encounter any year data in the
   * calendar file, assume the current year.
   */
  cyear = year;
  
  /*
   * Open a supplied calendar file (if any)
   */
  if (cfile != NULL) {
    if ((cfp = fopen(cfile, "r")) == NULL) {
      FPR(stderr, "pcal: can't open file: %s\n", cfile);
      exit(1);
    }
  }
  /*
   * Else see if a calendar file exists in the home directory
   */
  else if (nocal == 0 && (cp = getenv("HOME")) != NULL) {
    (void)strcpy(cbuf, cp);
    (void)strcat(cbuf, "/calendar");
    cfp = fopen(cbuf, "r");
  }
  /*
   * Write out PostScript prolog
   */
  PRT("%%! PS-Adobe-1.0\n");
  PRT("%%%%DocumentFonts:%s,%s,%s,%s\n",
      ascii_font, kanji_font, title_font, daynum_font);

  for (ap = pheader; *ap; ap++)
    PRT("%s\n", *ap);
  
  pmonth(m);
  
  return(0);
}

/*
 * pmonth - do calendar for month "m"
 */
pmonth(m)
int m;
{
  register char **s;
  register oldday = -1;
  register day;

  /* 
   * Set Fonts
   */
  PRT("/titlefont /%s def\n", title_font);
  PRT("/dayfont /%s def\n", daynum_font);
  PRT("/CompTextFont /%s /%s defineCompFont\n\n", ascii_font, kanji_font);

  /*
   * Do the calendar
   */
  PRT("/year %d def\n", year);
  PRT("/month %d def\n", m);
  PRT("printmonth\n\n");

  /*
   * Browse through the calendar file looking for day info
   */
  if ((day = getday(m)) == 0) {	/* no info */
    PRT("showpage\n");
    return;
  }
  do {
    if (day != oldday) {
      if (oldday == -1)
	PRT("%d [ \n", day);
      else
	PRT(" ] daytext %d [  \n", day);
      oldday = day;
    } 
    for (s = words; *s; s++) {
      putchar('(');
      ConvertString(*s);
      printf(")\n");
    }
  } while (day = getday(m));
  PRT("] daytext\n");
  PRT("showpage\n");
}

/*
 * getday - find next day entry for desired month in the calendar file
 */
getday(m)
     register m;
{
  static mon = 0;
  static eof = 0;
  register char *cp;
  register c;
  
  if (cfp == NULL)	/* whoops, no calendar file */
    return(0);
  
  if (m != mon) {		/* new month, rewind */
    rewind(cfp);
    eof = 0;
    mon = m;
  }
  if (eof)
    return(0);
 nextline:
  cp = lbuf;
  do {
    while ((c = getc(cfp)) != '\n' && c != EOF) {
      /* ignore leading white space */
      if (cp == lbuf && (c == ' ' || c == '\t'))
	continue;
      *cp++ = c;
    }
    if (c == EOF) {
      eof = 1;
      return(0);
    }
  } while (cp == lbuf);	/* ignore empty lines */
  *cp = 0;
  
  /* examine the line, see if its one we want */
  if ((c = parse(m)) == 0)
    goto nextline;
  
  return(c);
}

/*
 * parse - check calendar entry for desired month, break line into fields
 */
parse(m)
register m;
{
  register char *cp;
  register i;
  char ToLowerCase();

  cp = strtok(lbuf, " \t");	/* get first field */
  while (*cp) {
    *cp = ToLowerCase(*cp);
    cp++;
  }
  cp = lbuf;
  
  /*
   * Check for "year" line
   */
  if (strcmp(cp, "year") == 0) {
    cp = strtok((char *)0, " \t");
    if ((i = atoi(cp)) > 0) {
      if (i < 100)
	i += 1900;
      cyear = i;
    }
#ifdef DEBUG
    fprintf(stderr, "year: %d\n", cyear);
#endif DEBUG
    return(0);
  }
  /*
   * If field begins with alpha, try to decode month name
   */
#ifdef DEBUG
  fprintf(stderr, "month: %s\n", cp);
#endif
  if (isalpha(*cp)) {
    if (cyear != year)
      return(0);
    
    for (i = 0; months[i]; i++)
      if (strncmp(cp, months[i], 3) == 0) {
	if (++i != m)
	  return(0);
	
	/* month found, get day */
	
	if ((cp = strtok((char *)0, " \t")) == NULL)
	  return(0);
	if ((i = atoi(cp)) < 1 || i > 31)
	  return(0);
	if (loadwords())
	  return(i);
	return(0);
      }
    return(0);
  }
  /*
   * Not alpha month, try numeric
   */
#ifdef DEBUG
  fprintf(stderr, "day: %s\n", cp);
#endif
  if ((i = atoi(cp)) != m)
    return(0);
  while (isdigit(*cp))
    cp++;
  while (*cp && !isdigit(*cp))
    cp++;

  /* now get day */
  
  if ((i = atoi(cp)) < 1 || i > 31)
    return(0);
  
  /* Numeric dates may have a year */
#ifdef DEBUG
  fprintf(stderr, "year?: %s\n", cp);
#endif
  while (isdigit(*cp))
    cp++;
  while (*cp && !isdigit(*cp))
    cp++;
  if ((m = atoi(cp)) > 0) {
    if (m < 100)
      m += 1900;
    cyear = m;
  }
  
  if (cyear != year)
    return(0);
  
  if (loadwords())
    return(i);
  return(0);
}

/*
 * loadwords - tokenize line buffer into word array
 */
loadwords()
{
  register char **ap = words;
  register i;
  
  for (i = 0; *ap = strtok((char *)0, "\t"); ap++, i++);
#ifdef DEBUG
  fprintf(stderr, "loadwords:%s\n", ap);
#endif
  return(i);
}


#include <string.h>

char ToLowerCase(c)
     char c; 
{
  char *p;
  extern char *index();
  
  if( (p = index("ABCDEFGHIJKLMNOPQRSTUVWXYZ", c)) == NULL )
    return c;
  else
    return (char)(c - 'A' + 'a');
}

